@@ -1,3 +1,4 @@
+import os
 import logging
 import logging.config
 import re
@@ -5,7 +6,7 @@ import warnings
 from asyncio import get_event_loop, ensure_future, CancelledError
 from collections import deque, defaultdict
 from functools import partial
-from inspect import isawaitable, stack, getmodulename
+from inspect import getmodulename, isawaitable, signature, stack
 from traceback import format_exc
 from urllib.parse import urlencode, urlunparse
 from ssl import create_default_context, Purpose
@@ -22,10 +23,10 @@ from sanic.static import register as static_register
 from sanic.testing import SanicTestClient
 from sanic.views import CompositionView
 from sanic.websocket import WebSocketProtocol, ConnectionClosed
+import sanic.reloader_helpers as reloader_helpers
 
 
 class Sanic:
-
     def __init__(self, name=None, router=None, error_handler=None,
                  load_env=True, request_class=None,
                  strict_slashes=False, log_config=None,
@@ -86,12 +87,24 @@ class Sanic:
 
         :param task: future, couroutine or awaitable
         """
-        @self.listener('before_server_start')
-        def run(app, loop):
+        try:
             if callable(task):
-                loop.create_task(task())
+                try:
+                    self.loop.create_task(task(self))
+                except TypeError:
+                    self.loop.create_task(task())
             else:
-                loop.create_task(task)
+                self.loop.create_task(task)
+        except SanicException:
+            @self.listener('before_server_start')
+            def run(app, loop):
+                if callable(task):
+                    try:
+                        loop.create_task(task(self))
+                    except TypeError:
+                        loop.create_task(task())
+                else:
+                    loop.create_task(task)
 
     # Decorator
     def listener(self, event):
@@ -99,11 +112,26 @@ class Sanic:
 
         :param event: event to listen to
         """
+
         def decorator(listener):
             self.listeners[event].append(listener)
             return listener
+
         return decorator
 
+    def register_listener(self, listener, event):
+        """
+        Register the listener for a given event.
+
+        Args:
+            listener: callable i.e. setup_db(app, loop)
+            event: when to register listener i.e. 'before_server_start'
+
+        Returns: listener
+        """
+
+        return self.listener(event)(listener)
+
     # Decorator
     def route(self, uri, methods=frozenset({'GET'}), host=None,
               strict_slashes=None, stream=False, version=None, name=None):
@@ -131,12 +159,20 @@ class Sanic:
             strict_slashes = self.strict_slashes
 
         def response(handler):
-            if stream:
-                handler.is_stream = stream
-            self.router.add(uri=uri, methods=methods, handler=handler,
-                            host=host, strict_slashes=strict_slashes,
-                            version=version, name=name)
-            return handler
+            args = [key for key in signature(handler).parameters.keys()]
+            if args:
+                if stream:
+                    handler.is_stream = stream
+
+                self.router.add(uri=uri, methods=methods, handler=handler,
+                                host=host, strict_slashes=strict_slashes,
+                                version=version, name=name)
+                return handler
+            else:
+                raise ValueError(
+                    'Required parameter `request` missing'
+                    'in the {0}() route?'.format(
+                        handler.__name__))
 
         return response
 
@@ -184,7 +220,7 @@ class Sanic:
                           name=name)
 
     def add_route(self, handler, uri, methods=frozenset({'GET'}), host=None,
-                  strict_slashes=None, version=None, name=None):
+                  strict_slashes=None, version=None, name=None, stream=False):
         """A helper method to register class instance or
         functions as a handler to the application url
         routes.
@@ -197,9 +233,9 @@ class Sanic:
         :param strict_slashes:
         :param version:
         :param name: user defined route name for url_for
+        :param stream: boolean specifying if the handler is a stream handler
         :return: function or class instance
         """
-        stream = False
         # Handle HTTPMethodView differently
         if hasattr(handler, 'view_class'):
             methods = set()
@@ -267,7 +303,8 @@ class Sanic:
                     await fut
                 except (CancelledError, ConnectionClosed):
                     pass
-                self.websocket_tasks.remove(fut)
+                finally:
+                    self.websocket_tasks.remove(fut)
                 await ws.close()
 
             self.router.add(uri=uri, handler=websocket_handler,
@@ -278,13 +315,13 @@ class Sanic:
         return response
 
     def add_websocket_route(self, handler, uri, host=None,
-                            strict_slashes=None, name=None):
+                            strict_slashes=None, subprotocols=None, name=None):
         """A helper method to register a function as a websocket route."""
         if strict_slashes is None:
             strict_slashes = self.strict_slashes
 
         return self.websocket(uri, host=host, strict_slashes=strict_slashes,
-                              name=name)(handler)
+                              subprotocols=subprotocols, name=name)(handler)
 
     def enable_websocket(self, enable=True):
         """Enable or disable the support for websocket.
@@ -349,21 +386,26 @@ class Sanic:
     def static(self, uri, file_or_directory, pattern=r'/?.+',
                use_modified_since=True, use_content_range=False,
                stream_large_files=False, name='static', host=None,
-               strict_slashes=None):
+               strict_slashes=None, content_type=None):
         """Register a root to serve files from. The input can either be a
         file or a directory. See
         """
         static_register(self, uri, file_or_directory, pattern,
                         use_modified_since, use_content_range,
-                        stream_large_files, name, host, strict_slashes)
+                        stream_large_files, name, host, strict_slashes,
+                        content_type)
 
     def blueprint(self, blueprint, **options):
         """Register a blueprint on the application.
 
-        :param blueprint: Blueprint object
+        :param blueprint: Blueprint object or (list, tuple) thereof
         :param options: option dictionary with blueprint defaults
         :return: Nothing
         """
+        if isinstance(blueprint, (list, tuple)):
+            for item in blueprint:
+                self.blueprint(item, **options)
+            return
         if blueprint.name in self.blueprints:
             assert self.blueprints[blueprint.name] is blueprint, \
                 'A blueprint with the name "%s" is already registered.  ' \
@@ -416,7 +458,7 @@ class Sanic:
         uri, route = self.router.find_route_by_view_name(view_name, **kw)
         if not (uri and route):
             raise URLBuildError('Endpoint with name `{}` was not found'.format(
-                                view_name))
+                view_name))
 
         if view_name == 'static' or view_name.endswith('.static'):
             filename = kwargs.pop('filename', None)
@@ -470,7 +512,7 @@ class Sanic:
             specific_pattern = '^{}$'.format(pattern)
             supplied_param = None
 
-            if kwargs.get(name):
+            if name in kwargs:
                 supplied_param = kwargs.get(name)
                 del kwargs[name]
             else:
@@ -529,6 +571,10 @@ class Sanic:
 
         :return: Nothing
         """
+        # Define `response` var here to remove warnings about
+        # allocation before assignment below.
+        response = None
+        cancelled = False
         try:
             # -------------------------------------------- #
             # Request Middleware
@@ -544,6 +590,7 @@ class Sanic:
 
                 # Fetch handler from router
                 handler, args, kwargs, uri = self.router.get(request)
+
                 request.uri_template = uri
                 if handler is None:
                     raise ServerError(
@@ -554,6 +601,13 @@ class Sanic:
                 response = handler(request, *args, **kwargs)
                 if isawaitable(response):
                     response = await response
+        except CancelledError:
+            # If response handler times out, the server handles the error
+            # and cancels the handle_request job.
+            # In this case, the transport is already closed and we cannot
+            # issue a response.
+            response = None
+            cancelled = True
         except Exception as e:
             # -------------------------------------------- #
             # Response Generation Failed
@@ -564,24 +618,37 @@ class Sanic:
                 if isawaitable(response):
                     response = await response
             except Exception as e:
-                if self.debug:
+                if isinstance(e, SanicException):
+                    response = self.error_handler.default(request=request,
+                                                          exception=e)
+                elif self.debug:
                     response = HTTPResponse(
                         "Error while handling error: {}\nStack: {}".format(
-                            e, format_exc()))
+                            e, format_exc()), status=500)
                 else:
                     response = HTTPResponse(
-                        "An error occurred while handling an error")
+                        "An error occurred while handling an error",
+                        status=500)
         finally:
             # -------------------------------------------- #
             # Response Middleware
             # -------------------------------------------- #
-            try:
-                response = await self._run_response_middleware(request,
-                                                               response)
-            except BaseException:
-                error_logger.exception(
-                    'Exception occurred in one of response middleware handlers'
-                )
+            # Don't run response middleware if response is None
+            if response is not None:
+                try:
+                    response = await self._run_response_middleware(request,
+                                                                   response)
+                except CancelledError:
+                    # Response middleware can timeout too, as above.
+                    response = None
+                    cancelled = True
+                except BaseException:
+                    error_logger.exception(
+                        'Exception occurred in one of response '
+                        'middleware handlers'
+                    )
+            if cancelled:
+                raise CancelledError()
 
         # pass the response to the correct callback
         if isinstance(response, StreamingHTTPResponse):
@@ -604,7 +671,7 @@ class Sanic:
     def run(self, host=None, port=None, debug=False, ssl=None,
             sock=None, workers=1, protocol=None,
             backlog=100, stop_event=None, register_sys_signals=True,
-            access_log=True):
+            access_log=True, **kwargs):
         """Run the HTTP Server and listen until keyboard interrupt or term
         signal. On termination, drain connections before closing.
 
@@ -622,6 +689,13 @@ class Sanic:
         :param protocol: Subclass of asyncio protocol class
         :return: Nothing
         """
+        # Default auto_reload to false
+        auto_reload = False
+        # If debug is set, default it to true (unless on windows)
+        if debug and os.name == 'posix':
+            auto_reload = True
+        # Allow for overriding either of the defaults
+        auto_reload = kwargs.get("auto_reload", auto_reload)
 
         if sock is None:
             host, port = host or "127.0.0.1", port or 8000
@@ -634,16 +708,26 @@ class Sanic:
                 warnings.simplefilter('default')
             warnings.warn("stop_event will be removed from future versions.",
                           DeprecationWarning)
+        # compatibility old access_log params
+        self.config.ACCESS_LOG = access_log
         server_settings = self._helper(
             host=host, port=port, debug=debug, ssl=ssl, sock=sock,
             workers=workers, protocol=protocol, backlog=backlog,
-            register_sys_signals=register_sys_signals,
-            access_log=access_log)
+            register_sys_signals=register_sys_signals, auto_reload=auto_reload)
 
         try:
             self.is_running = True
             if workers == 1:
-                serve(**server_settings)
+                if auto_reload and os.name != 'posix':
+                    # This condition must be removed after implementing
+                    # auto reloader for other operating systems.
+                    raise NotImplementedError
+
+                if auto_reload and \
+                        os.environ.get('SANIC_SERVER_RUNNING') != 'true':
+                    reloader_helpers.watchdog(2)
+                else:
+                    serve(**server_settings)
             else:
                 serve_multiple(server_settings, workers)
         except BaseException:
@@ -683,12 +767,12 @@ class Sanic:
                 warnings.simplefilter('default')
             warnings.warn("stop_event will be removed from future versions.",
                           DeprecationWarning)
-
+        # compatibility old access_log params
+        self.config.ACCESS_LOG = access_log
         server_settings = self._helper(
             host=host, port=port, debug=debug, ssl=ssl, sock=sock,
             loop=get_event_loop(), protocol=protocol,
-            backlog=backlog, run_async=True,
-            access_log=access_log)
+            backlog=backlog, run_async=True)
 
         # Trigger before_start events
         await self.trigger_events(
@@ -733,7 +817,7 @@ class Sanic:
     def _helper(self, host=None, port=None, debug=False,
                 ssl=None, sock=None, workers=1, loop=None,
                 protocol=HttpProtocol, backlog=100, stop_event=None,
-                register_sys_signals=True, run_async=False, access_log=True):
+                register_sys_signals=True, run_async=False, auto_reload=False):
         """Helper function used by `run` and `create_server`."""
         if isinstance(ssl, dict):
             # try common aliaseses
@@ -774,9 +858,11 @@ class Sanic:
             'loop': loop,
             'register_sys_signals': register_sys_signals,
             'backlog': backlog,
-            'access_log': access_log,
+            'access_log': self.config.ACCESS_LOG,
             'websocket_max_size': self.config.WEBSOCKET_MAX_SIZE,
             'websocket_max_queue': self.config.WEBSOCKET_MAX_QUEUE,
+            'websocket_read_limit': self.config.WEBSOCKET_READ_LIMIT,
+            'websocket_write_limit': self.config.WEBSOCKET_WRITE_LIMIT,
             'graceful_shutdown_timeout': self.config.GRACEFUL_SHUTDOWN_TIMEOUT
         }
 
@@ -799,14 +885,16 @@ class Sanic:
 
         if self.configure_logging and debug:
             logger.setLevel(logging.DEBUG)
-        if self.config.LOGO is not None:
+
+        if self.config.LOGO is not None and \
+                os.environ.get('SANIC_SERVER_RUNNING') != 'true':
             logger.debug(self.config.LOGO)
 
         if run_async:
             server_settings['run_async'] = True
 
         # Serve
-        if host and port:
+        if host and port and os.environ.get('SANIC_SERVER_RUNNING') != 'true':
             proto = "http"
             if ssl is not None:
                 proto = "https"
