@@ -1,7 +1,9 @@
 import traceback
 from json import JSONDecodeError
-
 from sanic.log import logger
+from sanic.exceptions import MethodNotSupported
+from sanic.response import text
+
 
 HOST = '127.0.0.1'
 PORT = 42101
@@ -43,7 +45,7 @@ class SanicTestClient:
 
     def _sanic_endpoint_test(
             self, method='get', uri='/', gather_request=True,
-            debug=False, server_kwargs={},
+            debug=False, server_kwargs={"auto_reload": False},
             *request_args, **request_kwargs):
         results = [None, None]
         exceptions = []
@@ -54,6 +56,15 @@ class SanicTestClient:
                     results[0] = request
             self.app.request_middleware.appendleft(_collect_request)
 
+        @self.app.exception(MethodNotSupported)
+        async def error_handler(request, exception):
+            if request.method in ['HEAD', 'PATCH', 'PUT', 'DELETE']:
+                return text(
+                    '', exception.status_code, headers=exception.headers
+                )
+            else:
+                return self.app.error_handler.default(request, exception)
+
         @self.app.listener('after_server_start')
         async def _collect_response(sanic, loop):
             try:
